@@ -4,7 +4,7 @@ use warnings;
 
 package Git::Wrapper;
 {
-  $Git::Wrapper::VERSION = '0.028';
+  $Git::Wrapper::VERSION = '0.029';
 }
 #ABSTRACT: Wrap git(7) command-line interface
 
@@ -18,6 +18,7 @@ use File::pushd;
 use File::Temp;
 use IPC::Cmd        qw(can_run);
 use IPC::Open3      qw();
+use Scalar::Util    qw(blessed);
 use Sort::Versions;
 use Symbol;
 
@@ -26,34 +27,51 @@ use Git::Wrapper::File::RawModification;
 use Git::Wrapper::Log;
 use Git::Wrapper::Statuses;
 
-my $GIT = ( defined $ENV{GIT_WRAPPER_GIT} ) ? $ENV{GIT_WRAPPER_GIT} : 'git';
-
 sub new {
-  my ($class, $arg, %opt) = @_;
+  my $class = shift;
 
-  my $self = bless { dir => $arg, %opt } => $class;
+  # three calling conventions
+  # 1: my $gw = Git::Wrapper->new( $dir )
+  # 2: my $gw = Git::Wrapper->new( $dir , %options )
+  # 3: my $gw = Git::Wrapper->new({ dir => $dir , %options });
+
+  my $args;
+
+  if ( scalar @_ == 1 ) {
+    my $arg = shift;
+    if ( ref $arg eq 'HASH' ) { $args = $arg }
+    elsif ( blessed $arg )    { $args = { dir => "$arg" } }  # my objects, let me
+                                                             # show you them.
+    elsif ( ! ref $arg )      { $args = { dir =>  $arg  } }
+    else { die "Single arg must be hashref, scalar, or stringify-able object" }
+  }
+  else {
+    my( $dir , %opts ) = @_;
+    $dir = "$dir" if blessed $dir; # we can stringify it for you wholesale
+    $args = { dir => $dir , %opts }
+  }
+
+  my $self = bless $args => $class;
 
   die "usage: $class->new(\$dir)" unless $self->dir;
 
   return $self;
 }
 
-sub has_git_in_path { can_run('git') }
+sub AUTOLOAD {
+  my $self = shift;
 
-sub dir { shift->{dir} }
+  (my $meth = our $AUTOLOAD) =~ s/.+:://;
+  return if $meth eq 'DESTROY';
+
+  $meth =~ tr/_/-/;
+
+  return $self->RUN($meth, @_);
+}
 
 sub ERR { shift->{err} }
 sub OUT { shift->{out} }
 
-sub _opt {
-  my $name = shift;
-  $name =~ tr/_/-/;
-  return length($name) == 1
-    ? "-$name"
-    : "--$name"
-  ;
-}
-
 sub RUN {
   my $self = shift;
 
@@ -62,34 +80,9 @@ sub RUN {
 
   my $cmd = shift;
 
-  my $opt = ref $_[0] eq 'HASH' ? shift : {};
+  my( $parts , $stdin ) = _parse_args( $cmd , @_ );
 
-  my @cmd = $GIT;
-
-  my $stdin = delete $opt->{-STDIN};
-
-  for (grep { /^-/ } keys %$opt) {
-    (my $name = $_) =~ s/^-//;
-
-    my $val = delete $opt->{$_};
-    next if $val eq '0';
-
-    push @cmd, _opt($name) . _munge_val($name, $val);
-  }
-
-  push @cmd, $cmd;
-
-  for my $name (keys %$opt) {
-    my $val = delete $opt->{$name};
-    next if $val eq '0';
-
-    ( $name, $val ) = $self->_message_tempfile( $val )
-      if $self->_win32_multiline_commit_msg( $cmd, $name, $val );
-
-    push @cmd,  _opt($name) . _munge_val($name, $val);
-  }
-
-  push @cmd, @_;
+  my @cmd = ( $self->git , @$parts );
 
   my( @out , @err );
 
@@ -113,6 +106,9 @@ sub RUN {
 
     print STDERR join(' ',@cmd),"\n" if $DEBUG;
 
+    # Prevent commands from running interactively
+    local $ENV{GIT_EDITOR} = '';
+
     my $pid = IPC::Open3::open3($wtr, $rdr, $err, @cmd);
     print $wtr $stdin
       if defined $stdin;
@@ -127,7 +123,7 @@ sub RUN {
   print "status: $?\n" if $DEBUG;
 
   # In earlier gits (1.5, 1.6, I'm not sure when it changed), "git status"
-  # would exit 1 if there was nothing to commit, or in other cases.  This is
+  # would exit 1 if there was nothing to commit, or in other cases. This is
   # basically insane, and has been fixed, but if we don't require git 1.7, we
   # should cope with it. -- rjbs, 2012-03-31
   my $stupid_status = $cmd eq 'status' && @out && ! @err;
@@ -149,55 +145,6 @@ sub RUN {
   return @out;
 }
 
-sub _munge_val {
-  my( $name , $val ) = @_;
-
-  return $val eq '1'       ? ""
-    : length($name) == 1 ? $val
-    :                      "=$val";
-}
-
-sub _win32_multiline_commit_msg {
-  my ( $self, $cmd, $name, $val ) = @_;
-
-  return 0 if $^O ne "MSWin32";
-  return 0 if $cmd ne "commit";
-  return 0 if $name ne "m" and $name ne "message";
-  return 0 if $val !~ /\n/;
-
-  return 1;
-}
-
-sub _message_tempfile {
-  my ( $self, $message ) = @_;
-
-  my $tmp = File::Temp->new( UNLINK => 0 );
-  $tmp->print( $message );
-
-  return ( "file", '"'.$tmp->filename.'"' );
-}
-
-sub AUTOLOAD {
-  my $self = shift;
-
-  (my $meth = our $AUTOLOAD) =~ s/.+:://;
-  return if $meth eq 'DESTROY';
-
-  $meth =~ tr/_/-/;
-
-  return $self->RUN($meth, @_);
-}
-
-sub version {
-  my $self = shift;
-
-  my ($version) = $self->RUN('version');
-
-  $version =~ s/^git version //;
-
-  return $version;
-}
-
 sub branch {
   my $self = shift;
 
@@ -207,6 +154,18 @@ sub branch {
   return $self->RUN(branch => $opt,@_);
 }
 
+sub dir { shift->{dir} }
+
+sub git {
+  my $self = shift;
+
+  return $self->{git_binary} if defined $self->{git_binary};
+
+  return ( defined $ENV{GIT_WRAPPER_GIT} ) ? $ENV{GIT_WRAPPER_GIT} : 'git';
+}
+
+sub has_git_in_path { can_run('git') }
+
 sub log {
   my $self = shift;
 
@@ -227,11 +186,11 @@ sub log {
 
     my $current = Git::Wrapper::Log->new($1);
 
-    $line = shift @out; # next line;
+    $line = shift @out;         # next line;
 
     while ($line =~ /^(\S+):\s+(.+)$/) {
       $current->attr->{lc $1} = $2;
-      $line = shift @out; # next line;
+      $line = shift @out;       # next line;
     }
 
     die "no blank line separating head from message" if $line;
@@ -241,9 +200,9 @@ sub log {
     my $message = '';
     while (
       @out
-      and $out[0] !~ /^commit (\S+)/
-      and length($line = shift @out)
-    ) {
+        and $out[0] !~ /^commit (\S+)/
+          and length($line = shift @out)
+        ) {
       $line =~ s/^$initial_indent//; # strip just the indenting added by git
       $message .= "$line\n";
     }
@@ -266,37 +225,6 @@ sub log {
   return @logs;
 }
 
-sub supports_hash_object_filters {
-  my $self = shift;
-
-  # The '--no-filters' option to 'git-hash-object' was added in version 1.6.1
-  return 0 if ( versioncmp( $self->version , '1.6.1' ) eq -1 );
-  return 1;
-}
-
-sub supports_log_no_abbrev_commit {
-  my $self = shift;
-
-  # The '--no-abbrev-commit' option to 'git log' was added in version 1.7.6
-  return ( versioncmp( $self->version , '1.7.6' ) eq -1 ) ? 0 : 1;
-}
-
-sub supports_log_raw_dates {
-  my $self = shift;
-
-  # The '--date=raw' option to 'git log' was added in version 1.6.2
-  return 0 if ( versioncmp( $self->version , '1.6.2' ) eq -1 );
-  return 1;
-}
-
-sub supports_status_porcelain {
-  my $self = shift;
-
-  # The '--porcelain' option to git status was added in version 1.7.0
-  return 0 if ( versioncmp( $self->version , '1.7' ) eq -1 );
-  return 1;
-}
-
 my %STATUS_CONFLICTS = map { $_ => 1 } qw<DD AU UD UA DU AA UU>;
 
 sub status {
@@ -333,6 +261,124 @@ sub status {
   return $statuses;
 }
 
+sub supports_hash_object_filters {
+  my $self = shift;
+
+  # The '--no-filters' option to 'git-hash-object' was added in version 1.6.1
+  return 0 if ( versioncmp( $self->version , '1.6.1' ) eq -1 );
+  return 1;
+}
+
+sub supports_log_no_abbrev_commit {
+  my $self = shift;
+
+  # The '--no-abbrev-commit' option to 'git log' was added in version 1.7.6
+  return ( versioncmp( $self->version , '1.7.6' ) eq -1 ) ? 0 : 1;
+}
+
+sub supports_log_raw_dates {
+  my $self = shift;
+
+  # The '--date=raw' option to 'git log' was added in version 1.6.2
+  return 0 if ( versioncmp( $self->version , '1.6.2' ) eq -1 );
+  return 1;
+}
+
+sub supports_status_porcelain {
+  my $self = shift;
+
+  # The '--porcelain' option to git status was added in version 1.7.0
+  return 0 if ( versioncmp( $self->version , '1.7' ) eq -1 );
+  return 1;
+}
+
+sub version {
+  my $self = shift;
+
+  my ($version) = $self->RUN('version');
+
+  $version =~ s/^git version //;
+
+  return $version;
+}
+
+sub _message_tempfile {
+  my ( $message ) = @_;
+
+  my $tmp = File::Temp->new( UNLINK => 0 );
+  $tmp->print( $message );
+
+  return ( "file", '"'.$tmp->filename.'"' );
+}
+
+sub _munge_val {
+  my( $name , $val ) = @_;
+
+  return $val eq '1'       ? ""
+    : length($name) == 1 ? $val
+      :                      "=$val";
+}
+
+sub _opt {
+  my $name = shift;
+  $name =~ tr/_/-/;
+  return length($name) == 1
+    ? "-$name"
+      : "--$name"
+        ;
+}
+
+sub _parse_args {
+  my $cmd = shift;
+  die "initial argument must not be a reference\n"
+    if ref $cmd;
+
+  my( $stdin , @pre_cmd , @post_cmd );
+
+  foreach ( @_ ) {
+    if ( ref $_ eq 'HASH' ) {
+      $stdin = delete $_->{-STDIN}
+        if exists $_->{-STDIN};
+
+      for my $name ( keys %$_ ) {
+        my $val = delete $_->{$name};
+        next if $val eq '0';
+
+        if ( $name =~ s/^-// ) {
+          push @pre_cmd , _opt( $name ) . _munge_val( $name , $val );
+        }
+        else {
+          ( $name, $val ) = _message_tempfile( $val )
+            if _win32_multiline_commit_msg( $cmd, $name, $val );
+
+          push @post_cmd , _opt( $name ) . _munge_val( $name , $val );
+        }
+      }
+    }
+    elsif ( blessed $_ ) {
+      push @post_cmd , "$_";      # here be anteaters
+    }
+    elsif ( ref $_ ) {
+      die "Git::Wrapper command arguments must be plain scalars, hashrefs, "
+        . "or stringify-able objects.\n";
+    }
+    else { push @post_cmd , $_; }
+  }
+
+  return( [ @pre_cmd , $cmd , @post_cmd ] , $stdin );
+}
+
+sub _win32_multiline_commit_msg {
+  my ( $cmd, $name, $val ) = @_;
+
+  return 0 if $^O ne "MSWin32";
+  return 0 if $cmd ne "commit";
+  return 0 if $name ne "m" and $name ne "message";
+  return 0 if $val !~ /\n/;
+
+  return 1;
+}
+
 __END__
 
 =pod
@@ -343,7 +389,7 @@ Git::Wrapper - Wrap git(7) command-line interface
 
 =head1 VERSION
 
-version 0.028
+version 0.029
 
 =head1 SYNOPSIS
 
@@ -352,23 +398,62 @@ version 0.028
   $git->commit(...)
   print $_->message for $git->log;
 
+  # specify which git binary to use
+  my $git = Git::Wrapper->new({
+    dir        => '/var/foo ,
+    git_binary => '/path/to/git/bin/git' ,
+  });
+
 =head1 DESCRIPTION
 
 Git::Wrapper provides an API for git(7) that uses Perl data structures for
 argument passing, instead of CLI-style C<--options> as L<Git> does.
 
-=head1 METHODS
+=head1 METHOD INVOCATION
 
 Except as documented, every git subcommand is available as a method on a
-Git::Wrapper object.  Replace any hyphens in the git command with underscores.
+Git::Wrapper object. Replace any hyphens in the git command with underscores
+(for example, 'git init-db' would become C<$git->init_db>).
 
-The first argument should be a hashref containing options and their values.
-Boolean options are either true (included) or false (excluded).  The remaining
-arguments are passed as ordinary command arguments.
+=head2 Method Arguments
 
-  $git->commit({ all => 1, message => "stuff" });
+Methods accept a combination of hashrefs and scalars, which is used to build
+the command used to invoke git. Arguments passed in hashrefs will be
+automatically parsed into option pairs, but the ordering of these in the
+resulting shell command is not guarenteed (with the exceptino of options with
+a leading '-'; see below). Options that are passed as plain scalars will
+retain their order. Some examples may help clarify. This code:
 
-  $git->checkout("mybranch");
+  $git->commit({ message => "stuff" , all => 1 });
+
+may produce this shell command:
+
+  git commit --all --message="stuff"
+
+This code, however:
+
+  $git->commit(qw/ --message "stuff" / , { all => 1 });
+
+will always produce this shell command:
+
+  git commit --message "stuff" --all
+
+In most cases, this exact control over argument ordering is not needed and
+simply passing all options as part of a hashref, and all other options as
+additional list arguments, will be sufficient. In some cases, however, the
+ordering of options to particular git sub-commands is significant, resulting
+in the need for this level of control.
+
+I<N.b.> Options that are given with a leading '-' (with the exception of
+special options noted below) are applied as arguments to the C<git> command
+itself; options without a leading '-' are applied as arguments to the
+sub-command. For example:
+
+  $git->command({ -foo => 1 , bar => 2 });
+
+invokes the command line
+
+  git --foo=1 command --bar=2
 
 I<N.b.> Because of the way arguments are parsed, should you need to pass an
 explicit '0' value to an option (for example, to have the same effect as
@@ -384,8 +469,19 @@ Output is available as an array of lines, each chomped.
 
   @sha1s_and_titles = $git->rev_list({ all => 1, pretty => 'oneline' });
 
+=head3 Passing stringify-able objects as arguments
+
+Objects may be passed in the place of scalars, assuming those objects overload
+stringification in such a way as to produce a useful value. However, relying
+on this stringification is discouraged and likely to be officially deprecated
+in a subsequent release. Instead, if you have an object that stringifies to a
+meaningful value (I<e.g.>, a L<Path::Class> object), you should stringify it
+yourself before passing it to C<Git::Wrapper> methods.
+
+=head2 Error handling
+
 If a git command exits nonzero, a C<Git::Wrapper::Exception> object will be
-thrown.  It has three useful methods:
+thrown (via C<die()>). These objects have three useful methods:
 
 =over
 
@@ -403,11 +499,21 @@ the exit status
 
 =back
 
-The exception stringifies to the error message.
+These exception objects stringify to the error message.
+
+=head1 METHODS
 
 =head2 new
 
   my $git = Git::Wrapper->new($dir);
+  my $git = Git::Wrapper->new({ dir => $dir , git_binary => '/path/to/git' });
+
+  # To force the git binary location
+  my $git = Git::Wrapper->new($dir, 'git_binary' => '/usr/local/bin/git');
+
+=head2 git
+
+  print $git->git; # /path/to/git/binary/being/used
 
 =head2 dir
 
@@ -430,7 +536,7 @@ you want ANSI color highlighting, you'll need to bypass via the RUN() method
   my @logs = $git->log;
 
 Instead of giving back an arrayref of lines, the C<log> method returns a list
-of C<Git::Wrapper::Log> objects.  They have four methods:
+of C<Git::Wrapper::Log> objects. They have four methods:
 
 =over
 
@@ -505,8 +611,8 @@ There are four status groups, each of which may contain zero or more changes.
 
 =back
 
-Note that a single file can occur in more than one group.  Eg, a modified file
-that has been added to the index will appear in the 'indexed' list.  If it is
+Note that a single file can occur in more than one group. Eg, a modified file
+that has been added to the index will appear in the 'indexed' list. If it is
 subsequently further modified it will additionally appear in the 'changed'
 group.
 
@@ -514,12 +620,12 @@ A Git::Wrapper::Status object has three methods you can call:
 
   my $from = $status->from;
 
-The file path of the changed file, relative to the repo root.  For renames,
+The file path of the changed file, relative to the repo root. For renames,
 this is the original path.
 
   my $to = $status->to;
 
-Renames returns the new path/name for the path.  In all other cases returns
+Renames returns the new path/name for the path. In all other cases returns
 an empty string.
 
   my $mode = $status->mode;
@@ -631,15 +737,17 @@ with a successful command.
 
 On Win32 Git::Wrapper is incompatible with msysGit installations earlier than
 Git-1.7.1-preview20100612 due to a bug involving the return value of a git
-command in cmd/git.cmd.  If you use the msysGit version distributed with
+command in cmd/git.cmd. If you use the msysGit version distributed with
 GitExtensions or an earlier version of msysGit, tests will fail during
-installation of this module.  You can get the latest version of msysGit on the
+installation of this module. You can get the latest version of msysGit on the
 Google Code project page: L<http://code.google.com/p/msysgit/downloads>
 
 =head1 ENVIRONMENT VARIABLES
 
-Git::Wrapper normally uses the first 'git' binary in your path, but if the
-GIT_WRAPPER_GIT environment variable is set, that value will be used instead.
+Git::Wrapper normally uses the first 'git' binary in your path. The original
+override provided to change this was by setting the GIT_WRAPPER_GIT environment
+variable. Now that object creation accepts an override, you are encouraged to
+instead pass the binary location (git_binary) to new on object creation.
 
 =head1 SEE ALSO
 
